# -*- coding: utf-8 -*-

import sys
import os

curPath = os.path.dirname(os.path.realpath(__file__))
libPath = curPath + '/' + "3rdLib"
sys.path.append(libPath)

import logging
import json
from timeSerialApp import TimeSerialApp

_logger = logging.getLogger(__name__)
tsa = TimeSerialApp()


def parseTopic(topic):
	keyList = topic.split('/')
	
	pk = keyList[2]
	dn = keyList[3]
	attribute = keyList[6]
	
	return pk, dn, attribute  # attribute 为property 表示属性, 为其他值时表示事件名称


def procPayload(pk, dn, attribute, payload_json):
	global tsa
	
	if (attribute == "property"):
		for propertyName, propertyData in payload_json.items():
			tsa.pushPropertyInfo(pk, dn, propertyName, propertyData)
	
	else:
		tsa.pushEventInfo(pk, dn, attribute, payload_json["params"])


'''
# event参数是您调用函数时传入的数据，其类型是Buffer。数据结构可由您自行定义，当消息来自设备时，数据结构如下：
# event: { "topic" : ${TOPIC}, "payload" : ${PAYLOAD} "provider" : { "groupId" : xxx } }
        # TOPIC 为一个字符串类型，首字母必须是"/"，格式：
                # 当数据来源为设备属性时候： "/sys/${PRODUCT}/${DEVICE_NAME}/thing/event/property/post"
                # 当数据来源为设备事件时候： "/sys/${PRODUCT}/${DEVICE_NAME}/thing/event/${EVENT}/post"

        #PAYLOAD 其类型是Object。#设备属性上报payload格式：
                #{ "Power":{ "value":"on", "time":1524448722000 }, "WF":{ "value": {}, "time":1524448722000 } }

        #设备事件上报payload格式：
                #{"params":{"value":{"Power":"on","WF":"2"},"time":1524448722000}}
'''


def handler(event, context):
	if(_logger.isEnabledFor(logging.DEBUG)):
		_logger.debug(event)
	event_json = json.loads(event)
	if(event_json["topic"].startswith('/sys')):
		pk, dn, attribute = parseTopic(event_json['topic'])
		payload_json = json.loads(event_json["payload"])
	
		procPayload(pk, dn, attribute, payload_json)
	
	return 'hello world'
