#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# ====#====#====#====
# Author: jys
# File: store
# Time: 2019/10/8
# ====#====#====#====

import logging
import time
from influxdb import InfluxDBClient
import threading

_logger = logging.getLogger(__name__)


class Influxdb_Store(object):
	def __init__(self, host='localhost', port=8086, username='root', password='root', database="example", **kwargs):
		self.host = host
		self.port = port
		self.username = username
		self.password = password
		self.database = database
		self.kwargs = kwargs
		self.client = None
		self.dbLock = threading.Lock()
	
	def _storeToInfluxdb(self, dataDict):
		
		with self.dbLock:
			try:
				if (None == self.client):
					self.client = InfluxDBClient(self.host, self.port, self.username, self.password, self.database,
												 **self.kwargs)
					self.client.create_database(self.database)
				
				self.client.write_points(dataDict, time_precision='ms')
			
			except:
				self.client = None
				_logger.exception("Err")
		
	
	def _formatProperty(self, propertyInfo):
		'''
		
		:param propertyInfo:
		:return:
			 #  单值
			 {
				  "measurement": "temp",            // 属性名
				  "tags": {                         // 属性的标签
					"_productKey": "1234567890",    // pk
					"_deviceName": "0987654321",    // dn
					"_dataType"  : "property"       // 类型：property 表示属性， event 表示事件
				  },
				  "time": 1346846400000,            // 时间戳， 单位：ms
				  "fields": {
					"value": 123                    // 属性的值
				  }
			}
			
			或者
			
			#复合值
			{
				"measurement": "temp",            	// 属性名
				"tags": {                         	// 属性的标签
					"_productKey": "1234567890",    // pk
					"_deviceName": "0987654321",    // dn
					"_dataType"  : "property"       // 类型：property 表示属性， event 表示事件
				},
				"time": 1346846400000,            	// 时间戳， 单位：ms
				"fields": {
					"aa": 12,                       // 属性的值
					"bb": 12,                       // 属性的值
					"cc": 12,                       // 属性的值
					"dd": 12,                       // 属性的值
				}
			}
		
		'''
		
		dataDict = {}
		
		dataDict["measurement"] = propertyInfo["propertyName"]
		dataDict["tags"] = {
			"_productKey": propertyInfo["pk"],
			"_deviceName": propertyInfo["dn"],
			"_dataType": "property"
		}
		
		dataDict["time"] = propertyInfo["propertyData"]["time"]
		if (isinstance(propertyInfo["propertyData"]["value"], dict)):
			dataDict["fields"] = propertyInfo["propertyData"]["value"]
		elif (isinstance(propertyInfo["propertyData"]["value"], list)):
			_logger.warning("influxdb not support list type")
		else:
			dataDict["fields"] = {"value": propertyInfo["propertyData"]["value"]}
		
		return dataDict
	
	def _formatEvent(self, eventInfo):
		'''
		
		
		
		:param eventInfo:
		:return:
			{
				"measurement": "event123",        // 事件名
				"tags": {                         // 事件的标签
					"_productKey": "1234567890",    // pk
					"_deviceName": "0987654321",    // dn
					"_dataType"  : "event"          // 类型：property 表示属性， event 表示事件
				},
				"time": 1346846400000,            // 时间戳， 单位：ms
				"fields": {
					"speed" : 20.8,
					"level" : 4,
					"direction" : "East",
				}
			}
		
		
		'''
		dataDict = {}
		
		dataDict["measurement"] = eventInfo["eventName"]
		dataDict["tags"] = {
			"_productKey": eventInfo["pk"],
			"_deviceName": eventInfo["dn"],
			"_dataType": "event",
		}
		
		dataDict["time"] = eventInfo["eventData"]["time"]
		dataDict["fields"] = eventInfo["eventData"]["value"]
		
		return dataDict
	
	def storeProperty(self, propertyList):
		if (len(propertyList) == 0):
			return
			
		dataDict = [self._formatProperty(item) for item in propertyList]
		_logger.info("store property cnt: %s", len(propertyList))
		self._storeToInfluxdb(dataDict)
		
	def storeEvent(self, eventList):
		if (len(eventList) == 0):
			return
					
		dataDict = [self._formatEvent(item) for item in eventList]
		_logger.info("store event cnt data: %s", len(eventList))
		self._storeToInfluxdb(dataDict)

if (__name__ == "__main__"):
	format = '%(asctime)s--%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s'
	logging.basicConfig(level=logging.DEBUG, format=format)
	
	dbObj = Influxdb_Store()
	
	dataDict = [
		{
			"pk": "1234567890",
			"dn": "0987654321",
			"propertyName": "temp",
			"propertyData": {
				"time": int(time.time() * 1000),
				"value": int(str(int(time.time() * 1000))[:-5])
				
			}
		}
	]

	while(1):
		dbObj.storeProperty(dataDict)
		time.sleep(2)
		_logger.debug("time to query...")
		#result = dbObj.client.query('select * from temp where _productKey = $myTag;', bind_params={"myTag": "1234567890"})
		#for item in result:
		#	for x in item:
		#		print(x)
	
	# json_body = [
	# 	{
	# 		"measurement": "env",
	# 		"tags": {
	# 			"_productKey": "qwert",
	# 			"_deviceName": "asdf",
	# 			"_dataType": "property"
	# 		},
	# 		"time": int(time.time() * 1000),
	# 		"fields": {
	# 			"temp": str(int(time.time())),
	# 			"CO2": str(int(time.time()))
	# 		}
	# 	}
	# ]
	# dbObj.client.write_points(json_body, time_precision='ms')
	# result = dbObj.client.query('select * from env where time < now() -1d and _productKey = $pk;', bind_params={"pk": "qwert"})
	#
	# for item in result:
	# 	for x in item:
	# 		print(x)
	#
# print("Result: {0}".format(result))
