#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# ====#====#====#====
# Author: jys
# File: timeSerialApp
# Time: 2019/10/11
# ====#====#====#====


from store import Influxdb_Store
import monotonic
import threading
import thread
import logging
import time
import os

_logger = logging.getLogger(__name__)


def task_tsStore(tsa):
	while (1):
		
		# 数据进行周期性的存储
		if (False == tsa.isTimeOut()):
			_logger.info("looping ...")
			time.sleep(1)
			continue
		
		with tsa.lock:
			tsa.tsdbObj.storeProperty(tsa.propertyList)   # 存储所有属性数据
			tsa.propertyList = []
			
			tsa.tsdbObj.storeEvent(tsa.eventList)  		  #  存储所有事件数据
			tsa.eventList = []
		
		tsa.updateTimeOut()							      # 重置下次存储数据的时间


class TimeSerialApp(object):
	def __init__(self):
		self.eventList = []
		self.propertyList = []
		self.timeOut = monotonic.monotonic() + 2
		self.lock = threading.Lock()
		
		
		# 对接数据库的连接信息
		host = os.environ.get('host', 'localhost')
		port = os.environ.get('port', 8086)
		username = os.environ.get('username', 'root')
		password = os.environ.get('password', 'root')
		database = os.environ.get('database', 'example')
		self.tsdbObj = Influxdb_Store(host, port, username, password, database)
		
		thread.createThread(task_tsStore, (self,))
	
	def isTimeOut(self):
		'''
			判断是否到存储时间
			
		:return:
		'''
		
		return True if (self.timeOut < monotonic.monotonic()) else False
	
	def updateTimeOut(self):
		
		'''
			更新下次存储的时间
			
		:return:
		'''
		
		self.timeOut = monotonic.monotonic() + 2
	
	def pushEventInfo(self, pk, dn, eventName, eventData):
		'''
			将事件相关的数据以结构化的形式缓存
		:param pk:
		:param dn:
		:param eventName:
		:param eventData:
		:return:
		'''
		dataDict = {
			"pk": pk,
			"dn": dn,
			"eventName": eventName,
			"eventData": eventData
		}
		
		with self.lock:
			self.eventList.append(dataDict)
	
	def pushPropertyInfo(self, pk, dn, propertyName, propertyData):
		'''
			将属性相关的数据以结构化的形式缓存
			
		:param pk:
		:param dn:
		:param propertyName:
		:param propertyData:
		:return:
		'''
		
		dataDict = {
			"pk": pk,
			"dn": dn,
			"propertyName": propertyName,
			"propertyData": propertyData
		}
		
		with self.lock:
			self.propertyList.append(dataDict)
